;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                                                                     m
;       REBOOTAT.ASM
;
;       Reboot machine at given time.
;
;       Written by: Francesco Adinolfi  February 1992
;
;       Parameters: Time - "HH:MM" where HH and MM are 1 or 2 digits
;                   Time validated  HH <= 23 
;                                   0 <= MM <= 59
;                   All characters before 1st digit are ignored. 
;
;       Uses Interrupt 1C to monitor time.
;
;       The routine uses the current DOS time - the input time to 
;       calculate the number of ticks to elapse before reboot time (allowing 
;       for times past midnight.
;       The ISR then just decrements a 3 byte counter on each invocation (tick)
;       No registers are used by the ISR
;
;       N.B. THIS TSR USES A NON-STANDARD METHOD OF LOADING ITSELF.
;
;       It uses the DOS allocate memory call to find the lowest available 
;       memory area for installation (requires 48 bytes + MCB = 64),
;       loads the ISR code into this region and changes the Memory 
;       Control Block so that this is not released when the program exits. 
;       Therefore it uses a bare minimum of memory (often none for all practical
;       purposes, since it can install below the last loaded TSR). 
;       It also leaves no holes in memory.
;       However, since the method used is basically unsupported, I make no 
;       claims as to its operability on all systems, and is untested on versions
;       of DOS prior to 5.0. 
;       I am using the program to reboot several machines dedicated to modems for offsite users which 
;       need to run batch processing at night.
;
;       Assembled with Microsoft(c) MASM 5.10
;
;       Tested environments: 
;  486-33 AMI BIOS      386-33 AMI BIOS        286-10Mhz         286-10Mhz 
;  8 MB RAM             4 MB RAM               1MB RAM           640K RAM
;  DOS 6.0              DOS 5.0                DOS 5.0           DOS 5.0
;  386MAX               386MAX
;
;  Version 1.01 03/04/92        Fix problem of booting 1 hour early  
;          1.02 03/04/92        Display version number; add CRLF to messages
;          1.03 07/08/92        Fix problem of reboot at same HH:MM as current
;                               time causing reboot in approx. 1 hour.
;          1.04 05/13/93        Add CRLF to reboot message.     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.286
;
;       Define BIOS and ROM segments for symbolic addressing
;
bios_data segment at 40h                        ;For warm boot flag
     org     72h
warmboot     dw  ?
bios_data ends

rom_seg segment at 0F000h                       ;For warm boot jump
     org     0FFF0h
     assume cs:rom_seg                          ;(for clean assembly)
rom_boot:
rom_seg ends

;
;       Install code 
;
setup segment 'CODE'
     assume cs:setup, ds:setup
     org     2Ch
env_seg     dw   ?
     org    100h
start:     
     mov   ah,9
     mov   dx,offset version
     int   21h                                  ;Display version number
;
;    Get reboot time, and if valid install ISR.
;    All the following code is discarded when ISR installed.
;
                                                ;Get Time to reboot
     xor    ah,ah                               ;Flag getting HH
     xor    bx,bx                               ;Clear store register
     mov    si,80h                              ;Number of parameters
     mov    cl,byte ptr [si]
     xor    ch,ch
     inc    si
timechar:                                       ;While more chars to process
     lodsb                                      ;   next char from command line
     cmp    al,'0'                              ;   If digit
     jb     notdigit
     cmp    al,'9'
     ja     notdigit
     or     bl,bl                               ;      If third digit
     jnz    parmerr                             ;         Error exit
     mov    bl,bh                               ;      Shift in digit
     mov    bh,al
     jmp    short nextchar                      ;   else
notdigit:     
     or     ah,ah                               ;      If not HH
     jnz    parmerr                             ;         Error exit
     or     bh,bh                               ;      If 1st digit processed
     jz     nextchar                            
     cmp    al,':'                              ;         If not :
     jne    parmerr                             ;            Error exit
     or     bl,'0'                              ;         Ensure ASCII numeric
     mov    [hourstr],bx                        ;         and store in message
     mov    [boot_hh],bx                        ;         and process id. string
     xor    bx,bx                               ;         reset store reg.
     inc    ah                                  ;         Flag getting MM
nextchar:
     loop   timechar                            ;Endwhile
     or     ah,ah                               ;If HH only
     jz     parmerr                             ;   Error exit
     or     bh,bh                               ;If HH:      
     jz     parmerr                             ;   Error exit
     or     bl,'0'                              ;Ensure ASCII numeric 
     mov    [minstr],bx                         ;and store in message
     mov    [boot_mm],bx                        ;and process id. string
     jmp    short asc2bin                       ;Continue
parmerr:                                        ;Invalid parameter error exit
     mov    dx, offset errparm                   
     jmp    short disperr
asc2bin:     
     mov    ax,bx                               ;Get minutes
     and    ax,0F0Fh                            ;Convert to binary
     xchg   ah,al
     aad
     cmp    ax,59                               ;If MM > 59
     jg     mmerr                               ;   Error exit
     mov    bx,ax
     mov    ax,[hourstr]                        ;Get hours
     and    ax,0F0Fh                            ;Convert to binary
     xchg   ah,al
     aad
     cmp    ax,23                               ;If HH > 23
     jg     hherr                               ;   Error exit
     mov    cl,60                               ;Convert to minutes
     mul    cl
     add    bx,ax                               ;Total mins
     mov    ah,2Ch                              ;Now get current time
     int    21h
     mov    ax,60                               ;Convert to minutes
     mul    ch
     xor    ch,ch
     add    ax,cx                               
     mov    cl,dh                               ;Save seconds
     xchg   ax,bx
     sub    ax,bx                               ;Calculate minutes to go
     jg     calctick                            ;allowing for next day
     add    ax,24 * 60                          ;
     jmp    short calctick
mmerr:     
     mov    dx,offset errmm                     ;Invalid minutes
     jmp    short disperr
hherr:     
     mov    dx,offset errhh                     ;Invalid hours
disperr:     
     mov    ah,9                                ;Display error
     int    21h
     mov    ax,4C01h                            ;Return errorlevel 1
     int    21h
calctick:     
     mov    bx,1092                             ;Calculate ticks
     mul    bx                                  
     xchg   ax,cx
     mov    bl,18                               ;Subtract seconds
     mul    bl                                  ;as 18 ticks per sec
     xchg   ax,cx
     sub    ax,cx                               ;from total
     jnc    storectr
     dec    dx                                  ;Adjust high counter byte if necessary
storectr:     
     mov    [ctrlow],ax                         ;Store counter
     mov    [ctrhigh],dx
     mov    dx, offset msg                      ;Display reboot time message
     mov    ah,9
     int    21h
     mov    ax,351Ch                            ;Get Interrupt Vector 1C
     int    21h
     mov    [old_1C_IP],bx                      ;and store in ISR code
     mov    [old_1C_CS],es
     mov    es,[env_seg]                        ;Free environment space
     mov    ah,49h                              ;So that we can use it for the 
     int    21h                                 ;ISR if there are no holes in
                                                ;memory.
     mov    bx,(isrlen + 15)/ 16                ;Get # of paragraphs required
     mov    ah,48h                              ;Allocate memory area
     int    21h
     mov    es,ax                               ;Adress it
     mov    si,offset isr_code
     xor    di,di                               
     mov    cx, isrlen
     rep    movsb                               ;Relocate ISR code
     mov    bx,ax                               ;Segment below is the Memory
     dec    bx                                  ;Control Block
     mov    es,bx                               ;Address this
     mov    es:word ptr[1],ax                   ;Store segment address of ISR
     mov    di,8                                ;as owner and set process 
     mov    si,offset prog_name                 ;name to BOOTHHMM
     mov    cx,prog_name_len                    
     rep    movsb
     mov    ds,ax                               ;Segment of ISR start 
     mov    dx,offset isrstart - offset isr_code
     mov    ax,251Ch                            ;Set IV 1C to ISR start
     int    21h
     mov    ax,4C00h                            ;Terminate process 
     int    21h                                 ;(ISR stays resident)

version  db     "REBOOTAT Version 1.04",0Dh,0Ah,'$'
prog_name   db   "BOOT"                         ;Identify owner as BOOTMMHH
boot_hh     dw   ?
boot_mm     dw   ?
prog_name_len equ $ - prog_name
msg  db     "This machine will reboot at "
hourstr dw  ?
     db     ':'
minstr  dw  ?
     db 0Dh,0Ah
     db    '$'
errparm db   "Invalid time parameter must be 'HH:MM'" 
       db    "where HH and MM are 1 or 2 digits",0Dh,0Ah,'$'
errhh  db    "Hours must be < 23",0Dh,0Ah,'$'
errmm  db    "Minutes must be < 59",0Dh,0Ah,'$'
;
;       ISR code starts here
;
isr_code:
        assume  ds:nothing
;
;       This code relocates to offset zero within segment, so these
;       variable names are accessible only from the install code.
;
ctrlow   dw     ?                               ;3 byte counter
ctrhigh  dw     ?                               
old_1C_IP dw    ?
old_1C_CS dw    ?
isrstart:     
     dec    cs:word ptr[0]                      ;Decrement counter low
     jnz    continue                            ;If expired
     dec    cs:word ptr[2]                      ;   Decrement counter high
     jns    continue                            ;   if expired
     mov    ax,seg bios_data                    ;      Get BIOS Data segment
     mov    ds,ax                               
     assume ds:bios_data                        
     mov    warmboot,1234h                      ;      Set warm boot flag
     jmp    far ptr rom_boot                    ;      REBOOT
continue:     
     jmp    cs:dword ptr[4]                     ;call next in chain.
isrlen equ $ - isr_code
setup ends
end   start
